﻿using Infragistics.Win.UltraWinGrid;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ExcelExporterSampleHugeData
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            this.ultraGrid1.DataSource = BuildData(new int[] { 6, 4 ,2});
            this.ultraGridExcelExporter1.ExportFormattingOptions = Infragistics.Win.UltraWinGrid.ExcelExport.ExportFormattingOptions.None;         
        }
        #region Public Method - Data Building
        static Random rand = new Random(0);
        /// <summary>
        /// This will return a multiband dataset with as many bands as integer 
        /// values that you include in the numRows[] parameter. Each band will
        /// have that number of rows per parent. Such that 5, 4, 3 will have 5
        /// parent records, 4 child records per parent, which will in turn have
        /// 3 child records for each of it's records. For a total of 85 records
        /// (5 + (5 * 4) + ((5 * 4) * 3))
        /// </summary>
        /// <param name="numRows"></param>
        /// <returns></returns>
        static public DataSet BuildData(int[] numRows)
        {
            return BuildData(numRows, new int[] {});
        }

        /// <summary>
        /// This will return a multiband dataset with as many bands as integer 
        /// values that you include in the numRows[] parameter. Each band will
        /// have that number of rows per parent. Such that 5, 4, 3 will have 5
        /// parent records, 4 child records per parent, which will in turn have
        /// 3 child records for each of it's records. For a total of 85 records
        /// (5 + (5 * 4) + ((5 * 4) * 3))
        /// </summary>
        /// <param name="numRows"></param>
        /// <returns></returns>
        static public DataSet BuildData(int[] numRows, int[] numAddlCols)
        {
            DataSet ds = new DataSet("data");
            int numBands = numRows.Length;
            int addlCols;
            DataTable[] tables = new DataTable[numBands];

            for (int i = 0; i < numBands; i++)
            {
                tables[i] = new DataTable("table" + i.ToString());

                tables[i].Columns.Add("id" + i.ToString(), typeof(int));
                tables[i].Columns.Add("name", typeof(string));
                tables[i].Columns.Add("description", typeof(string));
                tables[i].Columns.Add("int1", typeof(int));
                tables[i].Columns.Add("decimal1", typeof(decimal));
                tables[i].Columns.Add("decimal2", typeof(decimal));
                tables[i].Columns.Add("datetime1", typeof(DateTime));
                tables[i].Columns.Add("datetime2", typeof(DateTime));
                tables[i].Columns.Add("checkbox", typeof(bool));

                ds.Tables.Add(tables[i]);
                if (i > 0)
                {
                    tables[i].PrimaryKey = new DataColumn[] { tables[i].Columns["id" + i.ToString()] };
                    tables[i].Columns.Add("parentId" + i.ToString(), typeof(int));
                    string relName = "rel_" + ((int)i - 1).ToString() + "-" + i.ToString();
                    ds.Relations.Add(relName, tables[i - 1].Columns["id" + ((int)i - 1).ToString()], tables[i].Columns["parentId" + i.ToString()]);
                }

                addlCols = i <= numAddlCols.Length - 1 ? numAddlCols[i] : 0;
                for (int acInd = 1; acInd <= addlCols; acInd++)
                    tables[i].Columns.Add("addlCol" + acInd.ToString(), typeof(string));
            }
            int crntId = 0;

            FillDataSet(ds, 0, numRows, -1, ref crntId, numAddlCols);
            return ds;
        }

        static private void FillDataSet(DataSet ds, int crntTableNum, int[] numRows, int parentId, ref int crntId, int[] numAddlCols)
        {
            DataRow dr;
            int addlCols;

            for (int j = 0; j < numRows[crntTableNum]; j++)
            {
                crntId += 1;

                dr = ds.Tables[crntTableNum].NewRow();
                dr["id" + crntTableNum.ToString()] = crntId;
                dr["name"] = string.Format("{0} {1}", (FirstName)rand.Next(0, 49), (Surname)rand.Next(0, 49));
                dr["description"] = "Description field for Table#" + crntTableNum.ToString() + " Row # " + j.ToString();
                dr["int1"] = rand.Next(0, 100);
                dr["decimal1"] = ((decimal)rand.Next(0, 10000)) / 100;
                dr["decimal2"] = ((decimal)rand.Next(0, 10000)) / 100;
                dr["datetime1"] = DateTime.Now.AddDays(crntTableNum * (-j));
                dr["datetime2"] = DateTime.Now.AddDays(-j);
                dr["checkbox"] = j % 2 == 0;


                addlCols = crntTableNum <= numAddlCols.Length - 1 ? numAddlCols[crntTableNum] : 0;
                for (int acInd = 1; acInd <= addlCols; acInd++)
                    dr["addlCol" + acInd.ToString()] = "Additional Data" + acInd.ToString();

                if (crntTableNum > 0)
                    dr["parentId" + crntTableNum.ToString()] = parentId;

                ds.Tables[crntTableNum].Rows.Add(dr);

                if (crntTableNum < numRows.Length - 1)
                    FillDataSet(ds, crntTableNum + 1, numRows, crntId, ref crntId, numAddlCols);
            }
        }

        #region Enum : FirstName
        enum FirstName
        {
            Jack = 0,
            Oliver,
            Charlie,
            Harry,
            Jacob,
            Alfie,
            Noah,
            Oscar,
            George,
            James,
            Thomas,
            Leo,
            Joshua,
            Logan,
            Freddie,
            Archie,
            Max,
            Henry,
            Ethan,
            Connor,
            William,
            Carter,
            Ryan,
            Michael,
            Malcolm,
            Sophia,
            Emma,
            Olivia,
            Ava,
            Isabella,
            Mia,
            Zoe,
            Lily,
            Emily,
            Madelyn,
            Madison,
            Chloe,
            Charlotte,
            Aubrey,
            Avery,
            Abigail,
            Kaylee,
            Layla,
            Harper,
            Ella,
            Amelia,
            Arianna,
            Riley,
            Aria,
            Hailey,
        }
        #endregion
        #region Enum : Surname
        enum Surname
        {
            Smith = 0,
            Johnson,
            Williams,
            Jones,
            Brown,
            Davis,
            Miller,
            Wilson,
            Moore,
            Taylor,
            Anderson,
            Thomas,
            Jackson,
            White,
            Harris,
            Martin,
            Thompson,
            Garcia,
            Martinez,
            Robinson,
            Clark,
            Rodriguez,
            Lewis,
            Lee,
            Walker,
            Hall,
            Allen,
            Young,
            Hernandez,
            King,
            Wright,
            Lopez,
            Hill,
            Scott,
            Green,
            Adams,
            Baker,
            Gonzalez,
            Nelson,
            Carter,
            Mitchell,
            Perez,
            Roberts,
            Turner,
            Phillips,
            Campbell,
            Parker,
            Evans,
            Edwards,
            Collins
        }
        #endregion

        #endregion

        private void btnExport_Click(object sender, EventArgs e)
        {
            this.ultraGridExcelExporter1.ExportAsync(this.ultraGrid1, "C:\\GridData.xlsx");
        }

        private void ultraGrid1_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            ultraGrid1.DisplayLayout.Override.HeaderClickAction = Infragistics.Win.UltraWinGrid.HeaderClickAction.SortSingle;
            foreach (UltraGridBand band in e.Layout.Bands)
                band.Indentation = 0;
        }
    }
}
